Unit cMagSecurity;
  {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
[==       unit cMagSecurity;
 Description:    VistA Imaging Network Server Security Component.
        For security reasons, Imaging workstations are not connected to the Image Servers
        The application uses this component to connect and disconnect as needed to
        the Image Servers. This is OS Specific, for Win32 calls.
        In a different OS, this component could be replaced by OS Specific code,
        and the Application could function with minimial changes needed.
        Any function that opens image files, will need this component.
==]
 Note:
  }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Stdctrls,
  Umagutils8,
  FileCtrl,
  Hash { , MagImageManager } {JMW 4/25/2005 p45}; //45

Type
  TMag4Security = Class(TComponent)
  Private
    FSecurityOn: Boolean;
    FDemoModeOn: Boolean;
    FUsername: String;
    FPassword: String;
    FConnectErrors: TStrings;
    FDisconnectErrors: TStrings;
    Fwriteaccesslist: TStrings;
    FShareList: TStrings;
    FGoodList: TStrings;
    FBadList: TStrings;
    FAccessPaths: TStrings;
    FMsgList: TStrings;
    FTestFile: TStrings;
    Procedure SetMsgList(Value: TStrings);
    Procedure SetShareList(Value: TStrings);
    Procedure SetAccessPaths(Value: TStrings);
    Procedure LoadConnectErrors;
    Procedure LoadDisConnectErrors;
    Function ErrorText(Code: Char; i: Integer): String;
    Procedure MsgToList(s: String);
//    function StrtoPchar(FromValue: string): Pchar;
    //45function ParseServerShare(filename: string): string;
    Function SecurityNeeded(Var Xmsg: String; Filename: String): Boolean;
    Function ConfirmUserNamePassword(Var Xmsg, SUserID, SPW: String; TmpUser, TmpPass, Servershare: String): Boolean;
    Function OSConnectToServer(Var Xmsg: String; SPW, SUserID, Servershare, Filename: String): Boolean;
    Procedure SetSecurityOn(Const Value: Boolean);

  Protected

  Public
    Constructor Create(AOwner: TComponent); Override; { Public declarations }
    Destructor Destroy; Override;

            {  Main call to open a secure connection to the server.}
            {      filename         : is the full path to the Image.
                   xmsg             : is returned
                   tmpUser, tmpPass : override the Published Propertys UserName and Password
                   createdir        : to allow/disallow creating new directories.}
    Function MagOpenSecurePath(Filename: String; Var Xmsg: String; TmpUser: String = ''; TmpPass: String = ''; CreateDir: Boolean = False): Boolean;

            {   Close all Open Connections to any Image Server}
    Function MagCloseSecurity(Var Xmsg: String; AddToList: Boolean = False): Boolean;

    Function ParseServerShare(Filename: String): String; // JMW 6/20/2005 p45 needed to use elsewhere

    Function SetNetUsernamePassword(Userpass: String; Var Xmsg: String): Boolean;
    //procedure GetGoodList(var t: Tstringlist);
    //procedure GetBadList(var t: Tstringlist);

  Published

            {  Retrieved from M.  Needed to make the connection to the Server}
    Property Password: String Read FPassword Write FPassword;
    Property Username: String Read FUsername Write FUsername;

            {  We don't check security when running in Demo Mode.}
    Property DemoModeOn: Boolean Read FDemoModeOn Write FDemoModeOn;

            {  We can turn off Network Security.  Mainly for use by System manager. Debugging}
    Property SecurityOn: Boolean Read FSecurityOn Write SetSecurityOn;
            {}
    Property ShareList: TStrings Read FShareList Write SetShareList;
    Property AccessPaths: TStrings Read FAccessPaths Write SetAccessPaths;

           {  List of messages compiled during current MagOpenSecurePath function call.
              Cleared after each call.}
    Property Msglist: TStrings Read FMsgList Write SetMsgList;

  End;

Procedure Register;

Implementation

//uses MagImageManager ;
{function netconnect2( lpnetresource: TNetResourceA; lpszPassword: LPSTR;
             lpszUsername: LPSTR; dwFlags: DWORD ):DWORD; external 'sconn32.dll' index 1;

function Netdisconnect2(lpzname: LPSTR; fwdConnection: DWORD;
             fForce: Boolean):DWORD; external 'sconn32.dll' index 2;
}

Constructor TMag4Security.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  FShareList := Tstringlist.Create;
  FGoodList := Tstringlist.Create;
  FBadList := Tstringlist.Create;
  Fwriteaccesslist := Tstringlist.Create;
  FAccessPaths := Tstringlist.Create;
  FConnectErrors := Tstringlist.Create;
  FDisconnectErrors := Tstringlist.Create;
  FMsgList := Tstringlist.Create;
  FSecurityOn := True;
  LoadConnectErrors;
  LoadDisConnectErrors;
  FTestFile := Tstringlist.Create;
  FTestFile.Add('  ** Start ** ');
  FTestFile.Add('Test file to prove ability to write to a directory');
  FTestFile.Add('  ** End ** ');
End;
{-------------------------------------------------------}

Destructor TMag4Security.Destroy;
Begin
  Fwriteaccesslist.Free;
  FShareList.Free;
  FGoodList.Free;
  FBadList.Free;
  FAccessPaths.Free;
  FConnectErrors.Free;
  FDisconnectErrors.Free;
  FMsgList.Free;
  FTestFile.Free;
  Inherited;
End;

{-------------------------------------------------------}
(*
function TMag4Security.StrtoPchar(FromValue: string): Pchar;
var size: integer;

begin
  size := length(FromValue);
  Result := stralloc(size + 1);
  StrPCopy(Result, FromValue);
end;   *)

{-------------------------------------------------------}

Function TMag4Security.MagOpenSecurePath(Filename: String; Var Xmsg: String; TmpUser: String = ''; TmpPass: String = ''; CreateDir: Boolean = False): Boolean;
Var
  Servershare: String;
  PW, UserID: String;
  Ymsg: String;
  Xcursor: TCursor;
  OSError : integer;
Begin
  Xcursor := Screen.Cursor;
  Try
    Screen.Cursor := crHourGlass;
  // Each attempt to connect creates a message list.
    FMsgList.Clear;
    MsgToList('Image Security for Filename: ' + Filename);
    Result := False;

  { Quit if Demo Mode, Security is OFF, or Image file is on Local/Mapped drive}
    If Not SecurityNeeded(Xmsg, Filename) Then
    Begin
      Result := True;
      Exit;
    End;
  { Get just the \\Server\share from the filename}
    Servershare := ParseServerShare(Filename);
  { Use Kernel utilities to Decrypt the UserName and Password.
    and return the Username and Password to use in this instance.}
    If Not ConfirmUserNamePassword(Xmsg, UserID, PW, TmpUser, TmpPass, Servershare) Then Exit;

    If (FBadList.Indexof(Servershare) > -1) Then
    Begin
      Xmsg := 'ERROR: ' + Servershare + ' is Not Accessible.';
      MsgToList(Xmsg);
      Exit;
    End;

  { Connect to the server/share using the OS Calls.{}
    MsgToList('OSConnectToServer Start : ' + DateTimeToStr(Now));
    If Not OSConnectToServer(Xmsg, PW, UserID, Servershare, Filename) Then
    Begin
      MsgToList('OSConnectToServer Fail  msg :  ' + Xmsg);
      MsgToList('OSConnectToServer Fail time :  ' + DateTimeToStr(Now));
      Exit;
    End;
    MsgToList('OSConnectToServer Success: ' + DateTimeToStr(Now));

  { Keep a GoodList of successful connections.  When MagCloseSecurity
     is called.  We'll disconnect from all in this list. }
    If (FGoodList.Indexof(Servershare) = -1) Then FGoodList.Add(Servershare);

  {If we are copying to an Image Server, we might have to create a new
     directory.  (The caller must have sent the 'createdir' parameter =TRUE
     for us to create a directory. }
    If (CreateDir And (Not Directoryexists(ExtractFileDir(Filename)))) Then
    Begin
     {$I-} // silence OS Error raising
      Forcedirectories(ExtractFileDir(Filename));
      {$I+}
      if (getlasterror <> 0) then MsgToList('GetLastError: ' + syserrormessage(getlasterror));
      If Not Directoryexists(ExtractFileDir(Filename)) Then
      Begin
        Xmsg := 'FATAL: Cannot create directory: ' + ExtractFileDir(Filename);
        MsgToList(Xmsg);
        MagCloseSecurity(Ymsg, True);
        MsgToList(Ymsg);
        Exit;
      End
      Else
        MsgToList('Directory created : ' + ExtractFileDir(Filename));
    End;
    Try
    { This is in case we can connect, but don't have access, this will
      force an Access is Denied Error. This usually happens when the
             user is already connected to the Network. We'll get a
             'Credential Conflict' error, we'll continue, but get access
              denied when trying to copy the image.
      Keep a list, so we only attempt writing to each directory once.  }
      If (CreateDir And (Fwriteaccesslist.Indexof(FUsername + '-' + ExtractFilePath(Filename)) = -1)) Then
      Begin
      {$I-} // silence OS Error raising
      FTestFile.SaveToFile(ExtractFilePath(Filename) + 'TestingWriteAccess.txt');
      {$I+}
      if (getlasterror <> 0) then
         begin
         msgToList('Error Testing Write Access to Image Server: ') ;
         xmsg :=  'OS Error: ' + syserrormessage(getlasterror);
         MsgToList('GetLastError: ' + xmsg);
         MsgToList(Xmsg + ' ' + ExtractFilePath(Filename));
         MagCloseSecurity(Ymsg, True);
         MsgToList(Ymsg);
         Exit;
         end;
      Application.Processmessages;
      MsgToList('Write Access tested OK to :' + ExtractFilePath(Filename));
      Fwriteaccesslist.Add(FUsername + '-' + ExtractFilePath(Filename));
      Application.Processmessages;
      DeleteFile(ExtractFilePath(Filename) + 'TestingWriteAccess.txt');
      End
    Except
      On e: Exception Do
      Begin
        Xmsg := 'FATAL: Exception writing to Image Server.';
        MsgToList(Xmsg + ' ' + ExtractFilePath(Filename));
        MsgToList('OS exception: ' + e.Message);
        MagCloseSecurity(Ymsg, True);
        MsgToList(Ymsg);
        Exit;
      End;
    End;
    Result := True;
    Xmsg := 'Success: Image Directory is accessible.';
    MsgToList(Xmsg + ' ' + Servershare);
  Finally
    Screen.Cursor := Xcursor;
  End;
End;

Function TMag4Security.MagCloseSecurity(Var Xmsg: String; AddToList: Boolean = False): Boolean;
Var
  xx: Word;
  i, Size: Integer;
  Lpzname: LPSTR;
  FdwConnection: DWORD;
  FForce: Boolean;
Begin
  { Usually, clear the list, and create new one.
    But when called from inside TMagSecurity we won't clear it, just add to it.}

  // JMW 4/25/2005 p45, if the cachemanager is caching, then stop this
   { TODO -oGarrett -cRefactor :
This MAGIMAGEMANAGER Can't be here.  MagSecurity is only required to know about the
     Connecting and Disconnecting to the Network. }

 //     if MagImageManager1.isImageCurrentlyCaching() then exit;

  If Not AddToList Then Msglist.Clear;
  FBadList.Clear;
  Result := True;
  If Not SecurityOn Then MsgToList('Security is OFF');
  If DemoModeOn Then MsgToList('Demo Mode, Security OFF');

  For i := FGoodList.Count - 1 Downto 0 Do
  Begin
    Size := (Length(FGoodList[i]));
    Lpzname := Stralloc(Size + 1);
    Strpcopy(Lpzname, FGoodList[i]);
    FdwConnection := 0;
    FForce := True;

    xx := WNetCancelConnection2(Lpzname, FdwConnection, FForce);

    StrDispose(Lpzname);
    If xx = 0 Then
    Begin
      MsgToList(' Disconnected : ' + FGoodList[i]);
      FGoodList.Delete(i)
    End
    Else
    Begin
      MsgToList(FGoodList[i] + '  ' + ErrorText('d', xx));
    End;

  End;
  If (FGoodList.Count > 0) Then
  Begin
    Result := False;
    Xmsg := '(' + Inttostr(FGoodList.Count) + ')' + ' Error disconnecting from Network Shares:';
    MsgToList(Xmsg);
    For i := 0 To FGoodList.Count - 1 Do
      MsgToList('- ' + FGoodList[i]);
    FGoodList.Clear; (*  should we be doing this  even for error  *)
  End
  Else
  Begin
    Xmsg := 'Disconnect Successful';
    MsgToList(Xmsg);
  End;
End;

Procedure TMag4Security.SetShareList(Value: TStrings);
Begin
  FShareList.Assign(Value);
End;

Procedure TMag4Security.SetAccessPaths(Value: TStrings);
Begin
  FAccessPaths.Assign(Value);
End;

Procedure TMag4Security.SetMsgList(Value: TStrings);
Begin
  FMsgList.Assign(Value);
End;

Procedure TMag4Security.MsgToList(s: String);
Begin
  FMsgList.Add('------   ' + s);
End;

Function TMag4Security.ErrorText(Code: Char; i: Integer): String;
Var
  j: Integer;
Begin
  Result := 'Unknown Error # : ' + Inttostr(i);
  If Code = 'c' Then
  Begin
    For j := 0 To FConnectErrors.Count - 1 Do
      If Copy(FConnectErrors[j], 1, Length(Inttostr(i))) = Inttostr(i) Then
      Begin
        Result := FConnectErrors[j];
        Break;
      End;
  End;
  If Code = 'd' Then
  Begin
    For j := 0 To FDisconnectErrors.Count - 1 Do
      If Copy(FDisconnectErrors[j], 1, Length(Inttostr(i))) = Inttostr(i) Then
      Begin
        Result := FDisconnectErrors[j];
        Break;
      End;
  End;

End;

Procedure Register;
Begin
  RegisterComponents('Imaging', [TMag4Security]);
End;

(*procedure TMag4Security.GetBadList(var t: Tstringlist);
begin
  t.assign(FBadlist);
end; *)

(*procedure TMag4Security.GetGoodList(var t: Tstringlist);
begin
  t.assign(FGoodlist);
end; *)

Function TMag4Security.ParseServerShare(Filename: String): String;
Begin
// IS There a OS Call for This ? ?
// get the \\Server\Share so we can connect to it.
  If Maglength(Filename, '\') > 5 Then
    Result := MagPiece(Filename, '\', 1) + '\'
      + MagPiece(Filename, '\', 2) + '\'
      + MagPiece(Filename, '\', 3) + '\'
      + MagPiece(Filename, '\', 4)
  Else
    Result := ExtractFilePath(Filename);
  // Get out any trailing backslashes ('\')
  While Copy(Result, Length(Result), 1) = '\' Do
    Result := Copy(Result, 1, Length(Result) - 1);
  MsgToList('\\Server\Share : ' + Result);
  msgToList('ExtractFilePath   \\Server\Share : '  + ExtractFilePath(filename));
End;

Function TMag4Security.SecurityNeeded(Var Xmsg: String; Filename: String): Boolean;
Var
  Msgoff: String;
  VApppath: String;
Begin
  VApppath := ExtractFilePath(Application.ExeName);
  Result := True;
  // Quit if Demo Mode, Security is OFF, or Image file is on Local/Mapped drive
  If Not FSecurityOn Then Msgoff := 'Image Security: OFF.  Attempt Canceled';
  If DemoModeOn Then Msgoff := 'Image Security CANCELED : Demo Mode is ON.';
  If (Uppercase(Copy(Filename, 2, 2)) = ':\') Then Msgoff := 'Image Security: NOT USED on Local/Mapped Drives';
  If (Uppercase(Copy(Filename, 1, 1)) = '.') Then Msgoff := 'Image Security: NOT USED on Local/Mapped Drives';
  If (Uppercase(Copy(Filename, 1, Length(VApppath))) = Uppercase(VApppath)) Then Msgoff := 'Image Security: NOT USED on Local/Mapped Drives';
  If (Msgoff <> '') Then
  Begin
    Xmsg := Msgoff;
    MsgToList(Xmsg);
    Result := False;
  End;
End;

Function TMag4Security.ConfirmUserNamePassword(Var Xmsg, SUserID, SPW: String; TmpUser, TmpPass, Servershare: String): Boolean;
Var
  i: Integer;
//45
//j : integer;
// value : string;
  Found: Boolean;
// aFile : TextFile;
//45
Begin
  Xmsg := 'Confirming UserName and Password...';
  Result := True;
  Found := False; //45
  Try
    {  Set the UserName Password to use. FUserName and FPassword are set
        from the Application by Setting the Published Properties: Password, Username
        The caller could send different UserName,Password as Parameters. }
    If (TmpUser <> '') Then
      SUserID := TmpUser
    Else
      SUserID := FUsername;
    If (TmpPass <> '') Then
      SPW := TmpPass
    Else
      SPW := FPassword;
    {   Stuart. In the Futuer, each NETWORK LOCATION entry could have it's own
         username and password, and a flag to say use or not use.}
    For i := ShareList.Count - 1 Downto 0 Do
    Begin
      If (Servershare = MagPiece(ShareList[i], '^', 2)) And (MagPiece(ShareList[i], '^', 5) <> '') Then
      Begin
        Found := True; //45
        {p135, send more messages}
        MsgToList('Server Share from List : ' + ShareList[i]);
        SUserID := MagPiece(ShareList[i], '^', 5);
        SPW := MagPiece(ShareList[i], '^', 6);
        If (SPW <> '') Then SPW := Decrypt(SPW);
        Break;
      End
    End;
  Except
    On Exception Do
    Begin
      Xmsg := 'ERROR: Invalid Site Password and/or UserName.  Contact IRM';
      MsgToList(Xmsg);
      Result := False;
      Exit;
    End;
  End;
        { //p45 this was commented out. looks like test code.
  if not found then
  begin
    assignfile(aFile, 'C:\output.txt');
    rewrite(aFile);
    writeln(afile, 'Looking for: ' + servershare);
    for i := 0 to sharelist.Count - 1 do
    begin
      writeln(afile, magpiece(sharelist[i], '^', 2) + ' - ' + magpiece(sharelist[i], '^', 5));
    end;
    closefile(afile);
  end;
  }
End;

Function TMag4Security.OSConnectToServer(Var Xmsg: String; SPW, SUserID, Servershare, Filename: String): Boolean;
Var
  Size: Integer;
  xx: Word;
  Lpnetresource: TNetResourceA;
  DwFlags: DWORD;
  XPw, XUserID: PChar;
  Ymsg: String;
Begin
  Result := False;
  Try
    Xmsg := 'Connecting to ' + Servershare + '...';

    //xPw := strtoPChar(spw);
    XPw := PChar(SPW);
    XUserID := PChar(SUserID);
    Lpnetresource.DwType := RESOURCETYPE_DISK;
    Lpnetresource.LpLocalName := Nil;
    Size := (Length(Servershare));
    Lpnetresource.LpRemoteName := Stralloc(Size + 1);
    Strpcopy(Lpnetresource.LpRemoteName, Servershare);
    Lpnetresource.LpProvider := Nil;
    DwFlags := 0;
  // Call the OS Function to connect to the ServerShare
    xx := WNetAddConnection2(Lpnetresource, XPw, XUserID, DwFlags);
  // Check for certain result codes:
    If xx = 1219 Then { credentail conflict, connect as self }
    Begin
      MsgToList('Error code: 1219, Credential conflict, continuing as current User...');
      XUserID := Nil;
      XPw := '';
      xx := WNetAddConnection2(Lpnetresource, XPw, XUserID, DwFlags);
    End;
    StrDispose(Lpnetresource.LpRemoteName);
 { TODO :
When Ruth called OpenSecureFile from her RAd Window, CPU would
Spike to 100, and stay there at this Application.processmessages.
command.  It would also happen if this  Application.processmessages
command was placed in her code. }
//  Application.processmessages; (* gek 12/16/96*)
    If Not (xx = 0) Then
    Begin
      MsgToList(' UserName: "' + SUserID + '"');
      Xmsg := 'FATAL: ' + ErrorText('c', xx) + '  ' + Filename;
      MsgToList(Xmsg);

   {'ERROR: Opening Image Security to the Image Directory. #'+inttostr(xx);}
      FBadList.Add(Servershare);
      Exit;
    End;
    Result := True;
  Except
    On e: Exception Do
    Begin
      Begin
        MsgToList(' UserName: "' + SUserID + '"');
        Xmsg := 'FATAL: Exception Connecting to Image Server.';
        MsgToList(Xmsg + ' ' + ExtractFilePath(Filename));
        MsgToList('OS Exception: ' + e.Message);
              // 03/11/02 IR Elsie : connection not being closed
        MagCloseSecurity(Ymsg, True);
        MsgToList(Ymsg);
      End;
    End;
  End;
End;

{----------------------------------------------------------}

Procedure TMag4Security.LoadConnectErrors;
Begin
{ this is a bad way to do things, it needs changed to use some kind of
  MSWindows function   }
  FConnectErrors.Add('999  Access to directory is Denied.');
  FConnectErrors.Add('0    NO_ERROR - success! ');
  FConnectErrors.Add('5    ERROR_ACCESS_DENIED { Access is denied. }');
  FConnectErrors.Add('85   ERROR_ALREADY_ASSIGNED { The local device name is already in use. }');
  FConnectErrors.Add('66   ERROR_BAD_DEV_TYPE { The network resource type is not correct. }');
  FConnectErrors.Add('1200 ERROR_BAD_DEVICE { The specified device name is invalid. }');
  FConnectErrors.Add('67   ERROR_BAD_NET_NAME { The network name cannot be found. }');
  FConnectErrors.Add('1206 ERROR_BAD_PROFILE { The network connection profile is corrupt. }');
  FConnectErrors.Add('1204 ERROR_BAD_PROVIDER { The specified network provider name is invalid. }');
  FConnectErrors.Add('170  ERROR_BUSY { The requested resource is in use. }');
  FConnectErrors.Add('1223 ERROR_CANCELLED');
  FConnectErrors.Add('1205 ERROR_CANNOT_OPEN_PROFILE { Unable to open the network connection profile. }');
  FConnectErrors.Add('1202 ERROR_DEVICE_ALREADY_REMEMBERED { An attempt was made to remember a device that had previously been remembered. }');
  FConnectErrors.Add('1208 ERROR_EXTENDED_ERROR { An extended error has occurred. }');
  FConnectErrors.Add('86   ERROR_INVALID_PASSWORD { The specified network password is not correct. }');
  FConnectErrors.Add('1203 ERROR_NO_NET_OR_BAD_PATH { No network provider accepted the given network path. }');
  FConnectErrors.Add('53 ERROR_NO_NET_OR_BAD_PATH { No network provider accepted the given network path. }');
  FConnectErrors.Add('1222 ERROR_NO_NETWORK { The network is not present or not started. }');
  FConnectErrors.Add('1219 ERROR_SESSION_CREDENTIAL_CONFLICT {The credentials supplied conflict with an existing set of credentials. }');
  FConnectErrors.Add('1326 ERROR_LOGON_FAILURE {The attempted logon in invalid.  This is due to either a bad user name or authentication information.}');
  FConnectErrors.Add('1784 ERROR_INVALID_USER_BUFFER {The supplied user buffer is invalid for the requested operation.}');
  FConnectErrors.Add('1785 ERROR_UNRECOGNIZED_MEDIA {The disk media is not recognized. It may not be formatted.}');
  FConnectErrors.Add('1786 ERROR_NO_TRUST_LSA_SECRET {The workstation does not have a trust secret.}');
  FConnectErrors.Add('1787 ERROR_NO_TRUST_SAM_ACCOUNT {The domain controller does not have an account for this workstation.}');
  FConnectErrors.Add('1788 ERROR_TRUSTED_DOMAIN_FAILURE {The trust relationship between the primary domain and the trusted domain failed.}');
  FConnectErrors.Add('1789 ERROR_TRUSTED_RELATIONSHIP_FAILURE {The trust relationship between this workstation and the primary domain failed.}');
  FConnectErrors.Add('1790 ERROR_TRUST_FAILURE {The network logon failed.}');
  FConnectErrors.Add('1792 ERROR_NETLOGON_NOT_STARTED {An attempt was made to logon, but the network logon service was not started.}');
  FConnectErrors.Add('1793 ERROR_ACCOUNT_EXPIRED {The user''s account has expired.}');
  FConnectErrors.Add('2202 The specified username is invalid.');
End;
   {----------------------------------------------------------}

Procedure TMag4Security.LoadDisConnectErrors;
Begin
  FDisconnectErrors.Add('0 NO_ERROR - success!');
  FDisconnectErrors.Add('1206  ERROR_BAD_PROFILE { The network connection profile is corrupt. }');
  FDisconnectErrors.Add('1205  ERROR_CANNOT_OPEN_PROFILE { Unable to open the network connection profile. }');
  FDisconnectErrors.Add('1204  ERROR_DEVICE_IN_USE { The device is in use by an active process and cannot be disconnected. }');
  FDisconnectErrors.Add('1208  ERROR_EXTENDED_ERROR { An extended error has occurred. }');
  FDisconnectErrors.Add('2250  ERROR_NOT_CONNECTED { This network connection does not exist. }');
  FDisconnectErrors.Add('2401  ERROR_OPEN_FILES { This network connection has files open or requests pending. }');
End;
{----------------------------------------------------------}

Procedure TMag4Security.SetSecurityOn(Const Value: Boolean);
Begin
  FBadList.Clear;
  FSecurityOn := Value;
End;

Function TMag4Security.SetNetUsernamePassword(Userpass: String; Var Xmsg: String): Boolean;
Var
  s: String;
Begin
  Try
    Xmsg := 'UserName^Password: ' + Userpass;
    If Userpass = '' Then
    Begin
      Result := True;
      Self.Username := '';
      Self.Password := '';
      Xmsg := 'UserName and Password are Null.';
      Exit;
    End;
    Self.Username := MagPiece(Userpass, '^', 1);
    s := MagPiece(Userpass, '^', 2);
    If (Self.Username = '') Or (s = '') Then
    Begin
      Xmsg := 'Invalid Imaging Network Username or Password' + #13 + 'Access to the Imaging Network Servers is disabled';
      Result := False;
    End
    Else
    Begin
      Self.Password := Decrypt(s);
      Result := True;
    End;
  Except
    On e: Exception Do
    Begin
      Xmsg := 'Error Setting UserName and Password: ' + e.Message;
      Result := False;
    End;
  End;
End;
End.
